#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

require 'puppet/face'
require 'entitlement/numbered_entry'

Puppet::Face.define(:emlicense, '0.0.1') do
  action(:list) do
    default

    summary 'List EM numbered licenses'

    description <<-DESC
      List the EM numbered licenses file.
    DESC

    arguments '<license_id> <module_name>'

    when_invoked do |*arguments|
      options = arguments.pop
      license_id = arguments[0] if arguments[0]
      module_name = arguments[1] if arguments[1]
      puppet_server = options[:puppetserver] || Puppet.settings[:server]
      entry = Entitlement::NumberedEntry.new([module_name], Date.today, Date.today + 1, 'file_name', 1, puppet_server, license_id)
      if license_id
        if module_name
          puts "Module : #{module_name}"
          entry.registered_nodes_for(module_name, puppet_server).each { |node| puts "  #{node}" }
        else
          puts "The following modules and nodes are registered on license_id #{license_id} and Puppet Server #{puppet_server}:"
          entry.registered_modules.each do |mname|
            puts "Module : #{mname}"
            entry.registered_nodes_for(mname, puppet_server).each { |node| puts "  #{node}" }
          end
        end
      else
        puts "The following licenses id's are registered:"
        entry.registered_licenses.each { |l| puts l }
      end
      nil
    end
  end
end
