#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

require 'easy_type'

Puppet::Type.type(:validation_results).provide(:puppetserver) do
  include EasyType::Provider

  defaultfor :in_control_running => false

  desc 'Report validation results through puppetserver'

  mk_resource_methods

  def validation_state_file
    "#{Puppet['statedir']}/validation_results.yaml".freeze
  end

  def insync?(_)
    update = if File.exist?(validation_state_file)
      last_results = File.mtime(validation_state_file).to_date
      #
      # Only write the results if the current state file is older then the requested update frequency
      #
      (Date.today - last_results).to_i >= resource.report_interval
    else
      true
    end
    File.write(validation_state_file, resource.results.to_yaml) if update
    resource.results.empty? || !update
  end

  #
  # Print a good change message when changing the contents of an array
  #
  def change_to_s(_, _)
    resource.results.to_json
  end

end
