module PuppetX
module EasyType
module Validation
#
# The implementation for the validatio check
#
module CheckImplementation

  def change_to_s(_current, _should)
    'The check has executed and found issues.'
  end

  def insync?(_to)
    if resource.noop
      Puppet.debug "#{path}: Skipping check because resource is set to noop."
      return true
    end
    return true if resource[:refreshonly].to_s == 'true'

    if !provider.validate_run?
      message = "Executable #{provider.executable} not found or not in path. Skipping this check..."
      Puppet.warning "#{path}: #{message}"
      resource.class.register_result(resource.control, 'SKIPPED', message)
      return true
    end
    if resource.skip.to_s == 'true'
      message = "Skipping this check at users request."
      Puppet.debug "#{path}: #{message}"
      resource.class.register_result(resource.control, 'SKIPPED', message)
      return true
    end
    output = provider.execute_check
    Puppet.debug output
    result = if resource.expected_value
      value_compare(output)
    else
      expected(output) && not_allowed(output)
    end
    message = result ? resource.title : resource.fail_message
    resource.class.register_result(resource.control, result ? 'PASSED' : "FAILED", message)
    resource.report_as.to_s.downcase == 'none' ? true : result
  end

  def value_compare(output)
    string_value = output.scan(resource.parse_value).flatten
    if string_value.size != 1
      Puppet.debug  "No valid value found after parsing output with #{resource.parse_value}"
      return false
    end
    actual_value = string_value.first.to_f
    Array(resource.expected_value).all?  do |comparison|
      final_comparison = "#{actual_value} #{comparison}"
      final_comparison = final_comparison.gsub('$value', string_value.first)
      value = eval(final_comparison)
      Puppet.debug "Executing compare '#{final_comparison}' return a '#{value}'"
      value
    end
  end

  def not_allowed(output)
    return true if resource[:not_allowed_output].nil?

    not_allowed_outputs = Array(resource[:not_allowed_output])
    not_allowed_outputs.all? do | not_allowed|
      if not_allowed.is_a? Regexp
        if output.match?(not_allowed)
          Puppet.debug "Regular expression '#{not_allowed}' found in output."
          return false
        else
          Puppet.debug "Regular expression '#{not_allowed}' NOT found in output."
          return true
        end
      else
        if output == not_allowed
          Puppet.debug "String '#{not_allowed}' found in output."
          return true
        else
          Puppet.debug "String '#{not_allowed}' NOT found in output."
          return false
        end
      end
    end
  end

  def expected(output)
    return true if resource[:expected_output].nil?

    expected_outputs = Array(resource[:expected_output])
    expected_outputs.all? do | expect|
      if expect.is_a? Regexp
        if output.match?(expect)
          Puppet.debug "Regular expression '#{expect}' found in output."
          return true
        else
          Puppet.debug "Regular expression '#{expect}' NOT found in output."
          return false
        end
      else
        if output == expect
          Puppet.debug "String '#{expect}' found in output."
          return true
        else
          Puppet.debug "String '#{expect}' NOT found in output."
          return false
        end
      end
    end
  end

end
end
end
end